/*
 * TismTool  Tis Veugen, 2010-2012
 *
 * File: User.cpp
 */

#include <cstring>
#include <cstdlib>
#include <boost/thread/thread.hpp>
#include "Tsm.h"
#include "User.h"

namespace Foreign
{
  UserClass::UserClass( TsmBase* oParent, string sInstanceName )
    : bCallback( false )
  {
    TsmPI__Port = new MyTsmIPort_ICbCtlrServer( this );
  }

  UserClass::~UserClass()
  {
    delete TsmPI__Port;
  }

  TsmInterfacePort* UserClass::GetProvidedInterface( string sPortName )
  {
    if ( sPortName == "UserPort" )
    {
      return TsmPI__Port;
    }
    string s = "No port '" + sPortName + "' @ UserClass";
    throw TsmException( s );
  }

  void UserClass::SetRequiredInterface( string sPortName, TsmInterfacePort* oProvidedInterface )
  {
    if ( sPortName == "UserPort" )
    {
      Port = static_cast<TsmIPort_ICtlrServer*>(oProvidedInterface);
      // Do NOT fill in Port->Source !!
      return;
    }
    string s = "No port '" + sPortName + "' @ UserClass";
    throw TsmException( s );
  }

  void UserClass::TsmRun( void* oObject )
  {
    this->iRv = this->Port->Start( true);

    // Wait for response from callback function.
    // Use global variable and non-busy polling for synchronisation.
    while ( !this->bCallback )
    {
      boost::posix_time::milliseconds ms(100);
      boost::this_thread::sleep( ms );
    }

    // Grant time to CbThread to finish its callback activity.
    boost::posix_time::milliseconds ms2(1000);
    boost::this_thread::sleep( ms2 );
  }

  void UserClass::CbStarted( float fCbStart )
  {
    Tsm::Log( "CbStarted" );
    this->bCallback = true;
  }


  UserForeign::UserForeign( TsmBase* oParent, string sInstanceName )
  {
    m_UserClass = new UserClass( this, "UserClass" );
  }

  UserForeign::~UserForeign()
  {
    delete m_UserClass;
  }

  TsmInterfacePort* UserForeign::GetProvidedInterface( string sPortName )
  {
    if ( sPortName == "d_rp_User" )
    {
      return m_UserClass->GetProvidedInterface( "UserPort" );
    }
    string s = "No port '" + sPortName + "' @ UserForeign";
    throw TsmException( s );
  }

  void UserForeign::SetRequiredInterface( string sPortName, TsmInterfacePort* oProvidedInterface )
  {
    if ( sPortName == "d_rp_User" )
    {
      m_UserClass->SetRequiredInterface( "UserPort", oProvidedInterface );
      return;
    }
    string s = "No port '" + sPortName + "' @ UserForeign";
    throw TsmException( s );
  }

  void UserForeign::TsmRun( void* oObject )
  {
    m_UserClass->TsmRun( oObject );
  }
}
