/*
 * TismTool © Tis Veugen, 2010-2012
 * Simulation, Verification
 *
 * TsmQueue.h
 * version 5
 */
#ifndef TSM_QUEUE_H
#define TSM_QUEUE_H

#include <deque>
#include <algorithm>
#include <boost/thread/mutex.hpp>
#include <boost/thread/locks.hpp>
#include "TsmEnvelope.h"
#include "TsmSupervisor.h"
using namespace std;

namespace TsmRTE
{
  class TsmQueue : public deque<TsmEnvelope>
  {
  private:
    unsigned int  mMaxQueueSize;
    boost::mutex  mLockMutex;
    int           mSeqId;

  public:
    TsmQueue()
      : mMaxQueueSize( 0 )
      , mLockMutex()
    {
    }

    int getMaxQueueSize()
    {
      return mMaxQueueSize;
    }

    int getSeqId()
    {
      return mSeqId;
    }

    void setSeqId( int SeqId )
    {
      mSeqId = SeqId;
    }

    void Put( TsmEnvelope& oEnvelope )
    {
      boost::unique_lock<boost::mutex> lock( mLockMutex );

      if ( TsmSupervisor::QueueOverflow( this->size() ) )
      {
        oEnvelope.getTrigger()->DBU();
      }
      else
      {
        this->push_back( oEnvelope );
        if ( mMaxQueueSize < this->size() )
        {
          mMaxQueueSize = this->size();
        }
      }
    }

    bool Dequeue( TsmEnvelope& oEnvelope )
    {
      boost::unique_lock<boost::mutex> lock( mLockMutex );

      if ( this->empty() )
      {
        return false;
      }

      oEnvelope = static_cast<TsmEnvelope>( this->front() );
      this->pop_front();
      return true;
    }

    void Empty()
    {
      for ( unsigned int i=0; i<this->size(); i++ )
      {
        this->at(i).getTrigger()->DBU();
      }
      this->clear();
    }

    string Snap()
    {
      deque<TsmEnvelope>::iterator it = this->begin();
      stringstream ss;

      while ( it != this->end() )
      {
        ss << (*it).Snap() + ",";
        it++;
      }
      return ss.str();
    }
  };
}
#endif
